//-----------------------------------------------------------------------------
// lottomachine.cpp
//
// Implementation for the lotto machine
//
// Midterm 
// Robert Mudge
// 10/10/1999
//
//-----------------------------------------------------------------------------

#include <iostream>
#include <ctime>
#include <cstdio>

#include "lottomachine.h"

namespace MIDTERM
{

  //---------------------------------------------------------------------------
  LottoMachine::LottoMachine(int number)
  {
    // initialize the object
    balls_picked = 0;

    #ifdef STDVECTOR    
      // initialize each ball with a number    
      if (number > 0)
      {
        number_of_balls = number;
        // create the balls in the lottomachine
        for (int i=0; i<number_of_balls; i++)
        {
          machine.push_back(Ball(i+1));
        }
      }
    #else
      // using a normal array of fixed size NUMBER_OF_BALLS
      number_of_balls = NUMBER_OF_BALLS;
      for (int i=0; i<number_of_balls; ++i)
      {
        machine[i].set_number(i+1);
      }
    #endif

    #ifdef LOTTOMACHINE_TEST
      std::cout << "Constructed LottoMachine of " << number << " balls\n";
    #endif
  }

  //---------------------------------------------------------------------------
  LottoMachine::~LottoMachine()
  {
    // cleanup the object
    // std::vector cleans up the ball objects
    #ifdef LOTTOMACHINE_TEST
      std::cout << "Destructed LottoMachine of " 
                << number_of_balls << " balls\n";
    #endif
  }
 
  //---------------------------------------------------------------------------
  const Ball& 
  LottoMachine::pick(void)
  {
    // pick a ball from contents 
    int tries = 0;
    int index = 0;
    static const int limit = 10 * number_of_balls;
 
    // initialize the random number generator
    static bool init = false;
    if (!init)
    {
      init = true;
      srand(time(0));
    }

    while (tries < limit)
    {
      index = rand() % number_of_balls;
      
      if (machine[index].pick())
      {
        // found a ball
        balls_picked++;
        break;
      }
      tries++;
    }

    if (tries == limit)
    {
      std::cerr << "Failed to pick a ball" << std::endl;
      index = 0;
    }

    return machine[index];
  }

  //---------------------------------------------------------------------------
  void  
  LottoMachine::reset(void)
  {
    // reset all ball to being not picked
    for(int i=0; i<number_of_balls; i++)
    {
      machine[i].reset();
    }
  }

  //---------------------------------------------------------------------------
  int  
  LottoMachine::get_balls_picked(void) const
  {
    // returns the number of ball picked from the machine
    return(balls_picked);
  }

  //---------------------------------------------------------------------------
  void
  LottoMachine::show(void) const
  {
    for(int i=0; i<number_of_balls; i++)
    {
      machine[i].show();
    }
  }

}

//---------------------------------------------------------------------------
#ifdef LOTTOMACHINE_TEST

#include <vector>

int
main ( void )
{
  // Unit test of the class LottoMachine
  // pick six balls and show each ball
  std::cout << "Unit test of class LottoMachine\n";

  #ifdef STDVECTOR
    std::cout << "\nUsing: std::vector<Ball> machine\n" << std::endl;
  #else
    std::cout << "\nUsing: Ball[NUMBER_OF_BALLS] machine\n" << std::endl;
  #endif

  const int NUMBER_OF_BALLS = 49;
  const int NUMBER_OF_PICKS = 6;

  std::vector<MIDTERM::Ball> card;

  MIDTERM::LottoMachine lm(NUMBER_OF_BALLS);

  while (lm.get_balls_picked() <= NUMBER_OF_PICKS)
  {
    card.push_back(lm.pick());
  }

  std::cout << "\nLotto Card" << std::endl;
  for(int index = 0; index < NUMBER_OF_PICKS; index++)
  {
    card[index].show();
  }

  std::cout << std::endl;

  return(0);
}

/* Example ouput
Unit test of class LottoMachine

Using: Ball[NUMBER_OF_BALLS] machine

Constructed LottoMachine of 49 balls

Lotto Card
Green Ball Number: 33
Green Ball Number: 43
Green Ball Number: 41
Green Ball Number: 16
Green Ball Number: 35
Green Ball Number: 34

Destructed LottoMachine of 49 balls
Press any key to continue
*/

#endif

// eof - lottomachine.cpp
